@@ -8,7 +8,7 @@ use serde::Deserialize;
 use std::fs;
 
 const DEFAULT_LUA_SCRIPT: &str = include_str!("init.lua");
-const UPGRADE_GUIDE_LINK: &str = "https://arijitbasu.in/xplr/en/upgrade-guide.html";
+const UPGRADE_GUIDE_LINK: &str = "https://xplr.dev/en/upgrade-guide.html";
 
 fn parse_version(version: &str) -> Result<(u16, u16, u16, Option<u16>)> {
     let mut configv = version.split('.');
@@ -133,25 +133,25 @@ mod test {
         assert!(check_version(VERSION, "foo path").is_ok());
 
         // Current release if OK
-        assert!(check_version("0.14.7", "foo path").is_ok());
+        assert!(check_version("0.15.2", "foo path").is_ok());
 
         // Prev major release is ERR
         // - Not yet
 
         // Prev minor release is ERR (Change when we get to v1)
-        assert!(check_version("0.13.7", "foo path").is_err());
+        assert!(check_version("0.14.2", "foo path").is_err());
 
         // Prev bugfix release is OK
-        assert!(check_version("0.14.6", "foo path").is_ok());
+        assert!(check_version("0.15.1", "foo path").is_ok());
 
         // Next major release is ERR
-        assert!(check_version("1.14.7", "foo path").is_err());
+        assert!(check_version("1.15.2", "foo path").is_err());
 
         // Next minor release is ERR
-        assert!(check_version("0.15.7", "foo path").is_err());
+        assert!(check_version("0.16.2", "foo path").is_err());
 
         // Next bugfix release is ERR (Change when we get to v1)
-        assert!(check_version("0.14.8", "foo path").is_err());
+        assert!(check_version("0.15.3", "foo path").is_err());
     }
 
     #[test]
