@@ -2,7 +2,7 @@
 
 use crate::app;
 use crate::cli::Cli;
-use crate::event_reader;
+use crate::event_reader::EventReader;
 use crate::explorer;
 use crate::lua;
 use crate::pipe_reader;
@@ -36,8 +36,9 @@ fn call_lua(
     _silent: bool,
 ) -> Result<Option<Vec<app::ExternalMsg>>> {
     let _focus_index = app
-        .directory_buffer()
-        .map(|d| d.focus())
+        .directory_buffer
+        .as_ref()
+        .map(|d| d.focus)
         .unwrap_or_default()
         .to_string();
 
@@ -48,8 +49,9 @@ fn call_lua(
 
 fn call(app: &app::App, cmd: app::Command, silent: bool) -> io::Result<ExitStatus> {
     let focus_index = app
-        .directory_buffer()
-        .map(|d| d.focus())
+        .directory_buffer
+        .as_ref()
+        .map(|d| d.focus)
         .unwrap_or_default()
         .to_string();
 
@@ -59,31 +61,34 @@ fn call(app: &app::App, cmd: app::Command, silent: bool) -> io::Result<ExitStatu
         (get_tty()?.into(), get_tty()?.into(), get_tty()?.into())
     };
 
-    Command::new(cmd.command().clone())
-        .env("XPLR_APP_VERSION", app.version())
-        .env("XPLR_PID", &app.pid().to_string())
-        .env("XPLR_INPUT_BUFFER", app.input_buffer().unwrap_or_default())
+    Command::new(cmd.command.clone())
+        .env("XPLR_APP_VERSION", app.version.clone())
+        .env("XPLR_PID", &app.pid.to_string())
+        .env(
+            "XPLR_INPUT_BUFFER",
+            app.input_buffer.clone().unwrap_or_default(),
+        )
         .env("XPLR_FOCUS_PATH", app.focused_node_str())
         .env("XPLR_FOCUS_INDEX", focus_index)
-        .env("XPLR_SESSION_PATH", app.session_path())
-        .env("XPLR_PIPE_MSG_IN", app.pipe().msg_in())
-        .env("XPLR_PIPE_SELECTION_OUT", app.pipe().selection_out())
-        .env("XPLR_PIPE_HISTORY_OUT", app.pipe().history_out())
+        .env("XPLR_SESSION_PATH", &app.session_path)
+        .env("XPLR_PIPE_MSG_IN", &app.pipe.msg_in)
+        .env("XPLR_PIPE_SELECTION_OUT", &app.pipe.selection_out)
+        .env("XPLR_PIPE_HISTORY_OUT", &app.pipe.history_out)
         .env("XPLR_MODE", app.mode_str())
-        .env("XPLR_PIPE_RESULT_OUT", app.pipe().result_out())
+        .env("XPLR_PIPE_RESULT_OUT", &app.pipe.result_out)
         .env(
             "XPLR_PIPE_GLOBAL_HELP_MENU_OUT",
-            app.pipe().global_help_menu_out(),
+            &app.pipe.global_help_menu_out,
         )
         .env(
             "XPLR_PIPE_DIRECTORY_NODES_OUT",
-            app.pipe().directory_nodes_out(),
+            &app.pipe.directory_nodes_out,
         )
-        .env("XPLR_PIPE_LOGS_OUT", app.pipe().logs_out())
+        .env("XPLR_PIPE_LOGS_OUT", &app.pipe.logs_out)
         .stdin(stdin)
         .stdout(stdout)
         .stderr(stderr)
-        .args(cmd.args())
+        .args(cmd.args)
         .status()
 }
 
@@ -141,13 +146,11 @@ impl Runner {
         // Why unsafe? See https://github.com/sayanarijit/xplr/issues/309
         let lua = unsafe { mlua::Lua::unsafe_new() };
         let mut app = app::App::create(self.pwd, &lua, self.config_file, self.extra_config_files)?;
-        app.config.general.set_read_only(self.read_only);
+        app.config.general.read_only = self.read_only;
 
-        fs::create_dir_all(app.session_path())?;
+        fs::create_dir_all(app.session_path.clone())?;
 
         let (tx_msg_in, rx_msg_in) = mpsc::channel();
-        let (tx_event_reader, rx_event_reader) = mpsc::channel();
-        let (tx_loop_waiter, rx_loop_waiter) = mpsc::channel();
         let (tx_pwd_watcher, rx_pwd_watcher) = mpsc::channel();
 
         app = app.explore_pwd()?;
@@ -163,16 +166,16 @@ impl Runner {
         };
 
         explorer::explore_recursive_async(
-            app.explorer_config().clone(),
-            app.pwd().into(),
+            app.explorer_config.clone(),
+            app.pwd.clone().into(),
             self.focused_path,
-            app.directory_buffer().map(|d| d.focus()).unwrap_or(0),
+            app.directory_buffer.as_ref().map(|d| d.focus).unwrap_or(0),
             tx_msg_in.clone(),
         );
-        tx_pwd_watcher.send(app.pwd().clone())?;
+        tx_pwd_watcher.send(app.pwd.clone())?;
 
         let mut result = Ok(None);
-        let session_path = app.session_path().to_owned();
+        let session_path = app.session_path.to_owned();
 
         term::enable_raw_mode()?;
 
@@ -182,7 +185,9 @@ impl Runner {
         // let mut stdout = stdout.lock();
         execute!(stdout, term::EnterAlternateScreen)?;
 
-        let mut fifo: Option<fs::File> = if let Some(path) = app.config().general().start_fifo() {
+        let mut fifo: Option<fs::File> = if let Some(path) = app.config.general.start_fifo.as_ref()
+        {
+            // TODO remove duplicate segment
             match start_fifo(path, &app.focused_node_str()) {
                 Ok(file) => Some(file),
                 Err(e) => {
@@ -196,7 +201,7 @@ impl Runner {
 
         let mut last_focus: Option<app::Node> = None;
 
-        let mut mouse_enabled = app.config().general().enable_mouse();
+        let mut mouse_enabled = app.config.general.enable_mouse;
         if mouse_enabled {
             if let Err(e) = execute!(stdout, event::EnableMouseCapture) {
                 app = app.log_error(e.to_string())?;
@@ -208,8 +213,9 @@ impl Runner {
         terminal.hide_cursor()?;
 
         // Threads
-        event_reader::keep_reading(tx_msg_in.clone(), rx_event_reader, tx_loop_waiter);
-        pwd_watcher::keep_watching(app.pwd(), tx_msg_in.clone(), rx_pwd_watcher)?;
+        pwd_watcher::keep_watching(app.pwd.as_ref(), tx_msg_in.clone(), rx_pwd_watcher)?;
+        let mut event_reader = EventReader::new(tx_msg_in.clone());
+        event_reader.start();
 
         // Enqueue on_load messages
         for msg in self.on_load {
@@ -220,7 +226,7 @@ impl Runner {
             match app.handle_task(task) {
                 Ok(a) => {
                     app = a;
-                    while let Some(msg) = app.pop_msg_out() {
+                    while let Some(msg) = app.msg_out.pop_front() {
                         use app::MsgOut::*;
                         match msg {
                             // NOTE: Do not schedule critical tasks via tx_msg_in in this loop.
@@ -235,14 +241,14 @@ impl Runner {
                             }
 
                             PrintPwdAndQuit => {
-                                result = Ok(Some(format!("{}\n", app.pwd())));
+                                result = Ok(Some(format!("{}\n", app.pwd)));
                                 break 'outer;
                             }
 
                             PrintFocusPathAndQuit => {
                                 result = Ok(app
                                     .focused_node()
-                                    .map(|n| format!("{}\n", n.absolute_path())));
+                                    .map(|n| format!("{}\n", n.absolute_path)));
                                 break 'outer;
                             }
 
@@ -272,29 +278,29 @@ impl Runner {
 
                             ExplorePwdAsync => {
                                 explorer::explore_async(
-                                    app.explorer_config().clone(),
-                                    app.pwd().into(),
-                                    app.focused_node().map(|n| n.relative_path().into()),
-                                    app.directory_buffer().map(|d| d.focus()).unwrap_or(0),
+                                    app.explorer_config.clone(),
+                                    app.pwd.clone().into(),
+                                    app.focused_node().map(|n| n.relative_path.clone().into()),
+                                    app.directory_buffer.as_ref().map(|d| d.focus).unwrap_or(0),
                                     tx_msg_in.clone(),
                                 );
-                                tx_pwd_watcher.send(app.pwd().clone())?;
+                                tx_pwd_watcher.send(app.pwd.clone())?;
                             }
 
                             ExploreParentsAsync => {
                                 explorer::explore_recursive_async(
-                                    app.explorer_config().clone(),
-                                    app.pwd().into(),
-                                    app.focused_node().map(|n| n.relative_path().into()),
-                                    app.directory_buffer().map(|d| d.focus()).unwrap_or(0),
+                                    app.explorer_config.clone(),
+                                    app.pwd.clone().into(),
+                                    app.focused_node().map(|n| n.relative_path.clone().into()),
+                                    app.directory_buffer.as_ref().map(|d| d.focus).unwrap_or(0),
                                     tx_msg_in.clone(),
                                 );
-                                tx_pwd_watcher.send(app.pwd().clone())?;
+                                tx_pwd_watcher.send(app.pwd.clone())?;
                             }
 
                             Refresh => {
                                 // $PWD watcher
-                                tx_pwd_watcher.send(app.pwd().clone())?;
+                                tx_pwd_watcher.send(app.pwd.clone())?;
                                 // UI
                                 terminal.draw(|f| ui::draw(f, &app, &lua))?;
                                 // Fifo
@@ -410,7 +416,7 @@ impl Runner {
                                     })
                                     .unwrap_or_else(|e| Err(e.to_string()));
 
-                                match pipe_reader::read_all(app.pipe().msg_in()) {
+                                match pipe_reader::read_all(&app.pipe.msg_in) {
                                     Ok(msgs) => {
                                         for msg in msgs {
                                             app = app.handle_task(app::Task::new(
@@ -435,7 +441,7 @@ impl Runner {
                                 execute!(terminal.backend_mut(), event::DisableMouseCapture)
                                     .unwrap_or_default();
 
-                                event_reader::pause_reading(&tx_event_reader, &rx_loop_waiter)?;
+                                event_reader.stop();
 
                                 terminal.clear()?;
                                 terminal.set_cursor(0, 0)?;
@@ -460,7 +466,7 @@ impl Runner {
                                 terminal.clear()?;
                                 term::enable_raw_mode()?;
                                 terminal.hide_cursor()?;
-                                event_reader::resume_reading(&tx_event_reader, &rx_loop_waiter)?;
+                                event_reader.start();
 
                                 if mouse_enabled {
                                     match execute!(
@@ -481,7 +487,7 @@ impl Runner {
                                 execute!(terminal.backend_mut(), event::DisableMouseCapture)
                                     .unwrap_or_default();
 
-                                event_reader::pause_reading(&tx_event_reader, &rx_loop_waiter)?;
+                                event_reader.stop();
 
                                 terminal.clear()?;
                                 terminal.set_cursor(0, 0)?;
@@ -499,7 +505,8 @@ impl Runner {
                                     })
                                     .unwrap_or_else(|e| Err(e.to_string()));
 
-                                match pipe_reader::read_all(app.pipe().msg_in()) {
+                                // TODO remove duplicate segment
+                                match pipe_reader::read_all(&app.pipe.msg_in) {
                                     Ok(msgs) => {
                                         for msg in msgs {
                                             app = app.handle_task(app::Task::new(
@@ -522,7 +529,7 @@ impl Runner {
                                 terminal.clear()?;
                                 term::enable_raw_mode()?;
                                 terminal.hide_cursor()?;
-                                event_reader::resume_reading(&tx_event_reader, &rx_loop_waiter)?;
+                                event_reader.start();
 
                                 if mouse_enabled {
                                     match execute!(
@@ -561,3 +568,13 @@ impl Runner {
         result
     }
 }
+
+/// Create a new runner object passing the default arguments
+pub fn runner() -> Result<Runner> {
+    Runner::new()
+}
+
+/// Create a new runner object passing the given arguments
+pub fn from_cli(cli: Cli) -> Result<Runner> {
+    Runner::from_cli(cli)
+}
